/*
        Product Revision  :  PIC18 MCUs - 1.81.7
        Device            :  PIC18F27K42
        Driver Version    :  2.00
    (c) 2018 Microchip Technology Inc. and its subsidiaries.  
*/

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdarg.h>
#include <string.h>
#include "mcc_generated_files/i2c1_master.h"
#include "mcc_generated_files/examples/i2c1_master_example.h"


#define _XTAL_FREQ 64000000


// CONFIG1L
#pragma config FEXTOSC = OFF    // External Oscillator Selection->Oscillator not enabled
#pragma config RSTOSC = HFINTOSC_64MHZ    // Reset Oscillator Selection->HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1

// CONFIG1H
#pragma config CLKOUTEN = ON    // Clock out Enable bit->CLKOUT function is enabled
#pragma config PR1WAY = ON    // PRLOCKED One-Way Set Enable bit->PRLOCK bit can be cleared and set only once
#pragma config CSWEN = ON    // Clock Switch Enable bit->Writing to NOSC and NDIV is allowed
#pragma config FCMEN = ON    // Fail-Safe Clock Monitor Enable bit->Fail-Safe Clock Monitor enabled

// CONFIG2L
#pragma config MCLRE = EXTMCLR    // MCLR Enable bit->If LVP = 0, MCLR pin is MCLR; If LVP = 1, RE3 pin function is MCLR 
#pragma config PWRTS = PWRT_OFF    // Power-up timer selection bits->PWRT is disabled
#pragma config MVECEN = ON    // Multi-vector enable bit->Multi-vector enabled, Vector table used for interrupts
#pragma config IVT1WAY = ON    // IVTLOCK bit One-way set enable bit->IVTLOCK bit can be cleared and set only once
#pragma config LPBOREN = OFF    // Low Power BOR Enable bit->ULPBOR disabled
#pragma config BOREN = SBORDIS    // Brown-out Reset Enable bits->Brown-out Reset enabled , SBOREN bit is ignored

// CONFIG2H
#pragma config BORV = VBOR_2P45    // Brown-out Reset Voltage Selection bits->Brown-out Reset Voltage (VBOR) set to 2.45V
#pragma config ZCD = OFF    // ZCD Disable bit->ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
#pragma config PPS1WAY = ON    // PPSLOCK bit One-Way Set Enable bit->PPSLOCK bit can be cleared and set only once; PPS registers remain locked after one clear/set cycle
#pragma config STVREN = ON    // Stack Full/Underflow Reset Enable bit->Stack full/underflow will cause Reset
#pragma config DEBUG = OFF    // Debugger Enable bit->Background debugger disabled
#pragma config XINST = OFF    // Extended Instruction Set Enable bit->Extended Instruction Set and Indexed Addressing Mode disabled

// CONFIG3L
#pragma config WDTCPS = WDTCPS_31    // WDT Period selection bits->Divider ratio 1:65536; software control of WDTPS
#pragma config WDTE = OFF    // WDT operating mode->WDT Disabled; SWDTEN is ignored

// CONFIG3H
#pragma config WDTCWS = WDTCWS_7    // WDT Window Select bits->window always open (100%); software control; keyed access not required
#pragma config WDTCCS = SC    // WDT input clock selector->Software Control

// CONFIG4L
#pragma config BBSIZE = BBSIZE_512    // Boot Block Size selection bits->Boot Block size is 512 words
#pragma config BBEN = OFF    // Boot Block enable bit->Boot block disabled
#pragma config SAFEN = OFF    // Storage Area Flash enable bit->SAF disabled
#pragma config WRTAPP = OFF    // Application Block write protection bit->Application Block not write protected

// CONFIG4H
#pragma config WRTB = OFF    // Configuration Register Write Protection bit->Configuration registers (300000-30000Bh) not write-protected
#pragma config WRTC = OFF    // Boot Block Write Protection bit->Boot Block (000000-0007FFh) not write-protected
#pragma config WRTD = OFF    // Data EEPROM Write Protection bit->Data EEPROM not write-protected
#pragma config WRTSAF = OFF    // SAF Write protection bit->SAF not Write Protected
#pragma config LVP = ON    // Low Voltage Programming Enable bit->Low voltage programming enabled. MCLR/VPP pin function is MCLR. MCLRE configuration bit is ignored

// CONFIG5L
#pragma config CP = OFF    // PFM and Data EEPROM Code Protection bit->PFM and Data EEPROM code protection disabled


#warning "use C90 for ultoa"


#define UART1_RX_BUFFER_SIZE 128
#define TMR2_INTERRUPT_TICKS  4  // ( 4 *8.192mS=> 32.7mS)
#define With_RTC_DS3231
#define DS3231_ADDR 0x68     // 7 bits adresse , sinon  0xD0  sur 8 bits
#include "RTC_DS3231_2021-04.h"

#define SQA_Dir        TRISAbits.TRISA3   
#define SQA            LATAbits.LATA3
#define RTC_Forcee     PORTBbits.RB1

#define OLED_ADDR 0x3C   // adresse 7 bits !
i2c1_address_t Addr=OLED_ADDR; 
// i2c1_address_t Adresse;
 i2c1_address_t ADXL345;

uint16_t i,j,k;
uint32_t L;
uint8_t c1,cx;
uint8_t temp=0;
volatile uint8_t Buffer1[UART1_RX_BUFFER_SIZE];
volatile uint16_t  Index1,i1;
volatile char Flag_Buffer1=0;

char TEXTE [128];
char *txt;
char *p1;
char *p2;
uint8_t  CRam1[64];

uint16_t Cpt_T2=0; 
uint16_t Cpt_T6=0; 
uint16_t Cpt_IT_RB0;

uint16_t EA1=0;
uint32_t  SMT1_Measure;
uint8_t *ST;
uint8_t  tmp[64];

bool I2C1_DevicePresent(int16_t address);
uint16_t  Bcd2Dec( uint8_t  val);
void BCD_To_Ascii( uint8_t  cc, uint8_t * px);
void UART1_Init(void);
void RAZ_UART1_Buffer1(void);
void PrintChar(uint8_t txData);
void Print (uint8_t * T1);
void CPrint (const uint8_t * T1);
void RAZ_UART1_Buffer1(void);
void PWM5_Init(void);
void PWM5_LoadDutyValue(uint16_t dutyValue);
void Init_Hardware(void);
void TMR2_Init(void);
void TMR4_Init(void);
void TMR6_Init(void);
void My_Init_ADC1(void);
void NCO1_Init (void);
void INTERRUPT_Init (void);
void EXT_INT_Init(void); 
void __interrupt(irq(U1RX),base(8)) UART1_rx_vect_isr(void);
void __interrupt(irq(TMR6),base(8),low_priority) TMR6_ISR(void);
void __interrupt(irq(TMR2),base(8),low_priority) TMR2_ISR(void);
void __interrupt(irq(INT0),base(8),low_priority)  INT0_ISR(void);
void __interrupt(irq(AD),base(8),low_priority) ADCC_ISR(void);
void __interrupt(irq(default),base(8),low_priority)  Default_ISR(void);



 
// rev 22/03/2021 
bool I2C1_DevicePresent(int16_t address)
{
  I2C1PIRbits.PCIF = 0;                // clear STOP bit flag
  I2C1ADB1 = address;                  // load slave addr and R/W
  I2C1CNT = 0;
  I2C1CON0bits.S = 1;                  // send START + addr
  while (I2C1PIRbits.PCIF == 0);       // wait for STOP                     
  return !I2C1CON1bits.ACKSTAT;
} 

// Conversion du code BCD en chiffre decimal
uint16_t  Bcd2Dec( uint8_t  val)
{
 return ( (val>>4*10) + (val%16) );
}

 void BCD_To_Ascii( uint8_t  cc, uint8_t * px)
 {
 *(px)=(cc>>4) + 48;
 *(px+1)=(cc & 0x0F) +48;
 *(px+2)=0;
 }

uint8_t Dec2Bcd(uint8_t n)
{
 return (n / 10 * 16 +  n % 10);
}
 
 
void UART1_Init(void)
{
    U1P1L = 0x00;
    U1P1H = 0x00;
    U1P2L = 0x00;
    U1P2H = 0x00;
    U1P3L = 0x00;
    U1P3H = 0x00;
    U1CON0 = 0xB0; // BRGS high speed; MODE Asynchronous 8-bit mode; RXEN enabled; TXEN enabled; ABDEN disabled; 
    // RXBIMD Set RXBKIF on rising RX input; BRKOVR disabled; 
    // WUE disabled; SENDB disabled; ON enabled; 
    U1CON1 = 0x80;    
    // TXPOL not inverted; FLO off; C0EN Checksum Mode 0; RXPOL not inverted;
    //RUNOVF RX input shifter stops all activity; STP Transmit 1Stop bit, receiver verifies first Stop bit; 
    U1CON2 = 0x00;
    U1BRGL = 0x8A; // BRGL 138; 
    U1BRGH = 0x00; // BRGH 0; 
    U1FIFO = 0x00; // STPMD in middle of first Stop bit; TXWRE No error; 
    // ABDIF Auto-baud not enabled or not complete; 
    // WUIF WUE not enabled by software; ABDIE disabled; 
    U1UIR = 0x00;
    // ABDOVF Not overflowed; TXCIF 0; RXBKIF No Break detected; 
    // RXFOIF not overflowed; CERIF No Checksum error; 
    U1ERRIR = 0x00;
    // TXCIE disabled; FERIE disabled; TXMTIE disabled; ABDOVE disabled;
    // CERIE disabled; RXFOIE disabled; PERIE disabled; RXBKIE disabled; 
    U1ERRIE = 0x00;
    PIE3bits.U1RXIE = 0;  // disable receive interrupt
}

 void PWM5_Init(void)
 {
     PWM5CON = 0x80;        // PWM5EN =1=enabled; PWM5POL=0=normal => active_hi;  
    PWM5DCH = 0x00;    //  Duty Cycle= 0    
    PWM5DCL = 0x00;   // 
    CCPTMRS1bits.P5TSEL = 2;  //page 360  Select timer 1=>TMR2  2=>TMR4 3=>TMR6
}
 
 
 void PWM5_LoadDutyValue(uint16_t dutyValue)
 {
     // Writing to 8 MSBs of PWM duty cycle in PWMDCH register
     PWM5DCH = (dutyValue & 0x03FC)>>2;
    // Writing to 2 LSBs of PWM duty cycle in PWMDCL register
     PWM5DCL = (dutyValue & 0x0003)<<6;
 }
  	

void TMR2_Init(void)
{
    // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T2CLKCON = 0x01;    // T2 CS 3..0 => FOSC/4     0x05 pour 500Khz  0x06  pour MFINTOSC (32 kHz)
    T2HLT = 0x00;
    T2RST = 0x00;  
    T2PR = 0xFF; // PR2 255; 
    T2TMR = 0x00;  // TMR2 0; 
    PIR4bits.TMR2IF = 0;  // Clearing IF flag.
  //  T2CON = 0x83; //  0x03=avec TMR2ON=0=OFF  0x83=avec TMR2ON=1=ON;  T2CKPS 1:1; T2OUTPS 1:4;  64uS
    T2CON = 0x60 | 0x07 ; //  Prescaler =1/64  Poscaler=1/8 => 1/128  ticks= 8192 uS
    PIE4bits.TMR2IE = 0;
}


void TMR4_Init(void)
{
     // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T4CLKCON = 0x01;    // T2CS FOSC/4     0x05 pour 500Khz
    T4HLT = 0x00;
    T4RST = 0x08;  // T4RSEL PWM5OUT; 
    T4PR = 0xFF; // PR4 255; 
    T4TMR = 0x00;  // TMR4 0; 
    PIR7bits.TMR4IF = 0;  // Clearing IF flag.
    T4CON = 0x60; //  avec TMR4ON=0=OFF    T2CKPS 1:64; T2OUTPS 1:1 not used
    T4CONbits.TMR4ON = 1;
   }
   

void TMR6_Init(void)
{
    T6CLKCON = 0x01;    // T6CS FOSC/4; 
    // T6PSYNC Not Synchronized; T6MODE Software control; T6CKPOL Rising Edge; T6CKSYNC Not Synchronized; 
    T6HLT = 0x00;
    T6RST = 0x00;    // T6RSEL T6CKIPPS pin; 
    T6PR = 0xFF;  // PR6 255; 
    T6TMR = 0x00;    // TMR6 0; 
    PIR9bits.TMR6IF = 0; // Clearing IF flag before enabling the interrupt.
    PIE9bits.TMR6IE = 1;  // Enabling TMR6 interrupt.
    T6CON = 0xE0; // T6CKPS 1:64; T6OUTPS 1:1; TMR6ON on;
    T6CONbits.TMR6ON = 0;
}

void EXT_INT_Init(void)
{
    PIR1bits.INT0IF = 0;  // Clear the interrupt flag 
    INTCON0bits.INT0EDG = 1;   // Set the external interrupt edge detect
    PIE1bits.INT0IE = 0;    
}


void  INTERRUPT_Init (void)
{
    INTCON0bits.IPEN = 1;
    bool state = (unsigned char)GIE;
    GIE = 0;
    IVTLOCK = 0x55;
    IVTLOCK = 0xAA;
    IVTLOCKbits.IVTLOCKED = 0x00; // unlock IVT
    IVTBASEU = 0;
    IVTBASEH = 0;
    IVTBASEL = 8;
    IVTLOCK = 0x55;
    IVTLOCK = 0xAA;
    IVTLOCKbits.IVTLOCKED = 0x01; // lock IVT

    GIE = state;
    // Assign peripheral interrupt priority vectors
    IPR0bits.IOCIP = 0;
    IPR3bits.U1TXIP = 0;
    IPR3bits.U1RXIP = 1;
    IPR1bits.INT0IP = 1;
    IPR3bits.I2C1EIP = 1;
    IPR2bits.I2C1RXIP = 1;
    IPR3bits.I2C1IP = 1;
    IPR3bits.I2C1TXIP = 1;
    IPR9bits.TMR6IP = 0;
    IPR1bits.ADIP = 0;
    IPR4bits.TMR2IP = 0;   
}


void SMT1_Init(void)
{ 
   //04-04-2021
     // registre  SMT1CON0
     SMT1CON0bits.WPOL=0; // Window Polarity
     SMT1CON0bits.SPOL=0; // Signal Polarity
     SMT1CON0bits.CPOL=0; // Clock Polarity
     SMT1CON0bits.SMT1PS1=0; // PS<1:0>: SMT Prescale Select bits = 1/1
     SMT1CON0bits.SMT1PS0=0;    // 11=1/8  10=1/4  01=1/2 00=1/1
     SMT1CON0bits.SMT1EN =1;    // SMT enabled
     SMT1CON0bits.SMT1STP=0;    // 0 =reset 1=STOP; 
     SMT1CON1bits.SMT1GO=0;     // 1= suit l'entr?   0=stop suivi
     SMT1CON1bits.SMT1REPEAT=0; // 0= Single  Acquisition   1=repeat mode
    // REGISTER 25-4: SMT1CLK: SMT CLOCK SELECTION REGISTER 16MHz
    //     000=FOSC/4    001=FOSC  010=HINTOSC 16MHz
      SMT1CLKbits.CSEL2=  0;
      SMT1CLKbits.CSEL1=  0;
      SMT1CLKbits.CSEL0=  1;
	// 25.7.1 PW (Pulse Width) AND PR (Periode Read)  ACQUISITIONI NTERRUPTS
      PIR1bits.SMT1PRAIF=0;
      PIE1bits.SMT1PRAIE=0;
     //  25.7.2  PERIOD MATCHI NTERRUPT
      PIR1bits.SMT1IF=0;
      PIE1bits.SMT1IE=0;
      // init  compteur
      SMT1PRU=0xFF;
      SMT1PRH=0xFF;
      SMT1PRL=0xFF;
     // MODE de fonctionnement TIMER  page 396  
     //   0000=Timer   0100=windowed measure  1000=counter
     //SMT1CON1=0;
     SMT1CON1bits.MODE3=0;
     SMT1CON1bits.MODE2=0;
     SMT1CON1bits.MODE1=0;
     SMT1CON1bits.MODE0=0;
  }

void SMT1_HaltCounter(void)
{
    SMT1CON0bits.SMT1STP = 1;
}

uint32_t SMT1_GetTimerValue()
{
    return (SMT1TMR);
}

void SMT1_SetPeriod(uint32_t periodVal)
{
    // Write to the SMT1 Period registers
    SMT1PRU = (uint8_t)(periodVal >> 16);
    SMT1PRH = (uint8_t)(periodVal >> 8);
    SMT1PRL = (uint8_t)periodVal;
}
void SMT1_DataAcquisitionEnable(void)
{
    // Start the SMT module by writing to SMTxGO bit
    SMT1CON1bits.SMT1GO = 1;
}

void SMT1_DataAcquisitionDisable(void)
{
    // Start the SMT module by writing to SMTxGO bit
    SMT1CON1bits.SMT1GO = 0;
}


void __interrupt(irq(U1RX),base(8)) UART1_rx_vect_isr()
{
   static char cx;
   // U1TXB='H';  // pour debugging only
  if ( (PIR3bits.U1RXIF==1) && (PIE3bits.U1RXIE ==1) )
  {   
      c1=U1RXB;
   //   U1TXB='*'; // echo
     if ((c1==13) || (i1>UART1_RX_BUFFER_SIZE-1))
     { 
      Flag_Buffer1=1;
         if(PIR3bits.U1RXIF) Buffer1[i1]=U1RXB;
      Buffer1[i1]=0;
      Index1=i1;
      i1=0; 
    // PIE3bits.U1RXIE =0;
     }
     else
     {
       Buffer1[i1]=c1;
       i1++;
       Index1=i1;
        if(PIR3bits.U1RXIF)
       {
       Buffer1[i1]=U1RXB;
       i1++;
       Index1=i1;  
       }
     } 
   } 
}




void __interrupt(irq(TMR6),base(8),low_priority) TMR6_ISR()
{  static volatile uint16_t TMR6_Ticks = 0;
     TMR6_Ticks++;
    if (TMR6_Ticks > 1000) {
        TMR6_Ticks = 0;
        Cpt_T6++;
    }
     PIR9bits.TMR6IF = 0;// clear the TMR6 interrupt flag
}


void __interrupt(irq(TMR2),base(8),low_priority) TMR2_ISR()
{
    static volatile uint16_t TMR2_Ticks = 0;
    // clear the TMR2 interrupt flag
    PIR4bits.TMR2IF = 0;
    if (++TMR2_Ticks  >= TMR2_INTERRUPT_TICKS)  // 4 => 32.768mS
    {
        Cpt_T2++;
        TMR2_Ticks = 0;
    }
}



void __interrupt(irq(INT0),base(8),low_priority)  INT0_ISR()
{
    PIR1bits.INT0IF = 0;
    Cpt_IT_RB0++;
   
}


void __interrupt(irq(AD),base(8),low_priority) ADCC_ISR()
{
    // Clear the ADCC interrupt flag
    PIR1bits.ADIF = 0;
 
}

void __interrupt(irq(default),base(8),low_priority)  Default_ISR()
{
      if(PIR3bits.U1TXIF)
      {
      U1TXB=temp++;
     }
}

void PrintChar(uint8_t txData)
{
    while(0 == PIR3bits.U1TXIF);
       U1TXB = txData;    // Write the data byte to the USART.
}

 void Print (uint8_t * Tc)
 { uint16_t k;
   k=0;      
   while(*(Tc+k)>0) 
   {
     while(PIR3bits.U1TXIF==0);
     U1TXB=*(Tc+k);
    k++;
   }
}
 
 
  void CPrint (const uint8_t* T1)
 { 
   while(*(T1)>0) 
   {
     while(PIR3bits.U1TXIF==0);
     U1TXB=*(T1);
    T1++;
   }
}

void CRLF1(void)
{
   while(PIR3bits.U1TXIF==0);
   U1TXB=13;
    while(PIR3bits.U1TXIF==0);
   U1TXB=10;  
  }    

void Init_Hardware(void)
{
    // LATx registers
    LATA = 0x00;
    LATB = 0x01;
    LATC = 0x80;
    // TRISx registers
    TRISA = 0b11100011;
    TRISB = 0xFF;
    TRISC = 0b10000000;
    // ANSELx registers
    ANSELC = 0;
    ANSELB = 0;
    ANSELA = 0x00;
    ANSELAbits.ANSELA1 = 1;
    // WPUx registers
    WPUE = 0x00;
    WPUB = 0x0F;
    WPUA = 0x00;
    WPUC = 0xC0;
    // RxyI2C registers
    RB1I2C = 0x00;
    RB2I2C = 0x00;
    RC3I2C = 0x00;
    RC4I2C = 0x00;
    // ODx registers
    ODCONA = 0x00;
    ODCONB = 0x00;
    ODCONC = 0x18;
    //  SLRCONx registers
    SLRCONA = 0b10111111;  // pas de slow rate sur RA6 Fosc/4 ouput
    SLRCONB = 0xFF;
    SLRCONC = 0xFF;
    // INLVLx registers
    INLVLA = 0xFF;
    INLVLB = 0xFF;
    INLVLC = 0xFF;
    INLVLE = 0x08;
    // PPS
    I2C1SDAPPS = 0x14;   //RC4->I2C1:SDA1;   
    RC2PPS = 0x26;   //RC2->NCO1:NCO1; 
    RC3PPS = 0x21;   //RC3->I2C1:SCL1;  
    RC4PPS = 0x22;   //RC4->I2C1:SDA1; 
    RC5PPS = 0x0D;   //RC5->PWM5:PWM5;    
    RC6PPS = 0x13;   //RC6->UART1:TX1;  
    INT0PPS = 0x08;   //RB0->EXT_INT:INT0;    
    SMT1WINPPS = 0x10;   //RC0->SMT1:SMT1WIN;    
    SMT1SIGPPS = 0x11;   //RC1->SMT1:SMT1SIG;    
    U1RXPPS = 0x17;   //RC7->UART1:RX1;    
    I2C1SCLPPS = 0x13;   //RC3->I2C1:SCL1;    
} 
    

void My_Init_ADC1(void)
{
ADCON0bits.FM = 1; //right justify
ADCON0bits.CS = 1; //ADCRC Clock
ADPCH = 0x01; //RA1 is Analog channel
TRISAbits.TRISA1 = 1; //Set RA0 to input
ANSELAbits.ANSELA1 = 1; //Set RA0 to analog
ADCON0bits.ON = 1; //Turn ADC On
}


void RAZ_UART1_Buffer1(void)
{   if (PIR3bits.U1RXIF==1)   c1=U1RXB ;
    for (i1=0;i1<sizeof(Buffer1);i1++) Buffer1[i1]=0;
      PIE3bits.U1IE  = 1;       // int uart1 enabled
      Flag_Buffer1 =0 ;
      U1CON0bits.U1RXEN=1;
      U1ERRIR = 0x00;
      U1ERRIE = 0x00;
      c1=0;
      Buffer1[0]=0;
      i1=0;
      Index1=0;
      c1=0;
      PIE3bits.U1RXIE =1;  
} 

void NCO1_Init(void)
{  // EN disabled; POL active_hi; PFM FDC_mode Fixed Duty Cycle =divide by 2; 
    NCO1CON = 0x00;
    NCO1CLK = 0x03;   // 00 = FOSC   0x03=500KHz
    NCO1ACCU = 0x00;
    NCO1ACCH = 0x00;
    NCO1ACCL = 0x00;
     // Increment
    NCO1INCU = 0x00;  // 1000Hz 4194
    NCO1INCH = 0x10;
    NCO1INCL = 0x62;
    NCO1CONbits.EN = 0; // Disable the NCO module
    //  NCO1CLK = 0x00;
    // FOSC/2= 32 000 000
    // NCO1INCR=1000   00 03 08
    // 1000/(2^20)=?0.00095367431640625?
    // 30517 Hz   32.768S
    //------
    // NCO1CLK = 0x03;
    // 500KHz/2= 250 0000
    // NCO1INCR= 4194  00h 10h 62h pour 1000Hz
   
    // NCO1CLK = 0x03;
    // 500KHz/2= 250 0000
    // NCO1INCR= 41943  00h A3h D7h pour 10KHz
     
}
 
void main(void)
{
    // Initialize the device
    // SYSTEM_Initialize();
    OSCCON1 = 0x60; // CSWHOLD may proceed; SOSCPWR Low power; 
    OSCCON3 = 0x00;
    OSCEN = 0x00;
    OSCFRQ = 0x08;   // HFFRQ 64_MHz; 
    OSCTUNE = 0x00;
    
    Init_Hardware();
    
    I2C1_Initialize();
    TMR6_Init();   // tick=1ms  1000x1=> 1sec
    My_Init_ADC1();
    TMR4_Init(); // pour  1KHz PWM5 
    SMT1_Init();
    PWM5_Init();  // sortie RC5  associ  TMR4
    TMR2_Init();  // tick=8192S *12 => 98.3mS
    EXT_INT_Init();
    UART1_Init();
    NCO1_Init ();
    
    putch('A'); putch('B');putch('C');putch(13);putch(10);
    txt=&TEXTE[0];
    Flag_Buffer1=0;
    sprintf(txt,"\r\nCompiled on %s at %s UTC by XC8 C90 version  %u\r\n",
            __DATE__, __TIME__, __XC8_VERSION);
    Print(txt);

    Cpt_T2=0;
    Cpt_T6=0;
    Cpt_IT_RB0=0;
    
    CPrint(" Init PWM5 out RC5 = 25%\r\n");
    PWM5_Init();
    PWM5_LoadDutyValue(250);
    
    CRLF1();
    PIE4bits.NCO1IE =0;
    SQA=1;
    NCO1CONbits.EN = 0; 
    NCO1ACCU = 0x00;
    NCO1ACCH = 0x00;
    NCO1ACCL = 0x00;
    SQA=0;
    NCO1INC =41943;    // 0x00A3D7   10Khz 
    sprintf(txt," Init NCO1 F=10Khz  U=%3d  H=%3d L=%3d soit 0X%06X ",NCO1INCU,NCO1INCH ,NCO1INCL,NCO1INC );
    Print(txt);CRLF1();
    NCO1CONbits.EN = 1; 
    CRLF1();
      
    INTERRUPT_Init ();
    T2CONbits.TMR2ON = 1;
    T6CONbits.TMR6ON = 1;
   
    CPrint(" Init My ADC 12bits entree EA1\r\n");
    My_Init_ADC1();
            
    SMT1_Measure=0;
    ST=&SMT1_Measure;
    SMT1_Measure=0;
    EA1=0;
    CPrint(" Start SMT1 pour 1 mesure ADC 12 bits \r\n");
    SMT1STATbits.RST=1;
    SMT1CON1bits.SMT1GO=1;
    //     SQA=1;
     ADCON0bits.GO = 1; //Start conversion
     while (ADCON0bits.GO); //Wait for conversion done
     EA1=(ADRESH<<8) + ADRESL;
     SMT1CON1bits.SMT1GO=0;
     txt=&TEXTE[0];
     SMT1_Measure= SMT1_GetTimerValue();
    // *(ST)=SMT1TMRL;
    // *(ST+1)=SMT1TMRH;
    // *(ST+2)=SMT1TMRU;
    // *(ST+3)=0;
     CPrint( " Stop SMT1   Nb Tics avec ultoa -> ");
     ultoa(txt,SMT1_Measure,10);
     Print(txt);
     sprintf(txt," , soit  %lu uS",SMT1_Measure>>6);
     Print(txt);    CRLF1();
    
     txt=&TEXTE[0];
    *(txt)=0; 
    CRLF1(); 
    CPrint(" Test Presence I2C device (adressage 8 bits)\r\n"); 
    I2C1CON2bits.ABD=0; //  ADB=0; // Transmitted address data is loaded from the I2CxADB0/1 registers.      
    I2C1CON0bits.EN = 1;
     for (i=2; i<=253; i=i+2)
    {
     if (I2C1_DevicePresent(i)) 
     {   
       k=i>>1;
       sprintf(txt," @ decimal #%4u  soit @Device7bits = 0X%02X ",i,k);
       switch (k)
       {
        case 0x27: 
        case 0x28: 
        case 0x29: 
        case 0x2A: 
        // case etc ..   
            strcat(txt,"PCF8574 8bits I/O");// de base + 7 autres
            break; 
        case 0x37:
            strcat(txt,"LCD 2119 2x16cars"); // de base + 7 autres
            break;  
        case 0x38:
        case 0x39:
        case 0x3A:
        case 0x3B:
        // case etc ..                                 
            strcat(txt,"PCF8574A 8bits I/O"); // de base + 7 autres
            break;  
        case 0x3C:
            strcat(txt,"OLED SSD1306");
            break;
        case 0x53:
            strcat(txt,"ADXL345 board");
            break;            
        case 0x57:
            strcat(txt,"EEPROM 512bytes");
            break;
        case 0x68:
            strcat(txt,"RTC DS3231");
            break;
        default:
            strcat(txt,"non recens");        
            break;
       }   
       Print(txt);CRLF1();
     }
    }
   I2C1CON0bits.EN = 0;
   I2C1CON2bits.ABD=1; 
   CRLF1();

    CPrint("Test IVT interrupt  18F27K42\r\n");
    PIE3bits.U1RXIE =1;
    PIE4bits.TMR2IE = 1;
    PIE9bits.TMR6IE = 1;
    PIE0bits.IOCIE = 1;
            
    INTCON0bits.GIEH = 1;
    INTCON0bits.GIEL = 1;

   #ifdef With_RTC_DS3231
   CPrint(" #1 Active sortie SQW=1Hz: \r\n");
   CPrint(" GO\r\n");
   // set SQW output  Start oscillateur
   tmp[0]=0x40; // data 
   I2C1_Write1ByteRegister(DS3231_ADDR,0x0E, tmp[0]);
   CPrint(" ok\r\n");
   __delay_ms(100);

   for (i=1;i<32;i++) tmp[i]=0;
   CPrint( " Etat pin RB1 = ");
   PrintChar((PORTB & 0x02)+48);
   CRLF1();
    
   if(RTC_Forcee==0)    // RB1 input
   {
    CPrint( " Init RTC Forcee a samedi 02/03/21 18:46:57 via  RB1=0 \r\n");
    tmp[0]=0x00;  //start register
     tmp[1]=0x57;  //second
      tmp[2]=0x16;  //write min
       tmp[3]=0x18; //write hour;
        tmp[4]=0x06;  ///write day of week
         tmp[5]=0x02;    // write date
          tmp[6]=0x03;      // write month
           tmp[7]=0x21;    // write year
            tmp[8]=0x00;
     SQA=1;
     p1=&tmp[0];
     I2C1_WriteNBytes(DS3231_ADDR,p1,8);
     SQA=0;
    }
    __delay_ms(1000);
    CPrint(" Re-Lecture et affichage des 7 Registres DS3231 \r\n");
    j=0;
    do
    {
    CPrint(" Registre #");
    putchar(j+48);
     putchar(9);
    cx=Lecture_Affichage_Registre_DS3231(j,1);
    CRLF1();
    j++;
    }
    while(j<8) ;
    CRLF1();
   Print_Date_Complete();
    __delay_ms(1000);
 
   #else
   CPrint(" Pas de gestion RTC DS3231\r\n");
   #endif
    
   j=0;
   while (1)
   {
        RAZ_UART1_Buffer1();
        __delay_ms(1000);
        // Add your application code
        if (Flag_Buffer1==1)
        {
           PIE3bits.U1RXIE = 0;
           CPrint("Recu :\r\n");
           Print(Buffer1);
           CRLF1();
        p2=&Buffer1[0];
       if (  (*(p2)=='U') && (*(p2+1)==';') && ( *(p2+19)==';') )
       { //U;03;03;21;15;14;03;#
        CPrint("\r\n MAJ RTC \r\n");
       // Print(Buffer1);    CRLF1();
        MAJ_RTC();
        CRLF1();
        p2=0;
        }
        if ((*(p2)=='H') && (*(p2+1)=='e') && ( *(p2+2)=='l') && ( *(p2+3)=='p') )
       {
        CPrint("\r\n Help Commandes :\r\n");
        CPrint(" MAJ RTC  \r\n");
        CPrint(" ex: U;03;03;21;15;14;03;#  pour 03 mars 2021 , 15H14  Mercredi \r\n");
        CPrint(" Help \r\n");
        Print(Buffer1);    CRLF1();
          p2=0 ;
         }  
           PIE3bits.U1RXIE = 1;
        }
          ADCON0bits.GO = 1; //Start conversion
          while (ADCON0bits.GO); //Wait for conversion done
          EA1=(ADRESH<<8 ) + ADRESL;
       
        k=Cpt_T2 & 0x03FF;
        SQA=1;
        PWM5_LoadDutyValue(k);
        Display_Date_Time(1);
        SQA=0;
        sprintf(txt,"  Cpt_T2 %5d   Cpt_T6=%5d  Cpt IT_RBO=%5d PWM5 out= %3d %s EA1=%5d  \r\n ",Cpt_T2,Cpt_T6,Cpt_IT_RB0, k/10," %",EA1);
        Print(txt);
        j++;
        if(j>60)
        {
            j=0;
            CRLF1();
        }
    }
}
/**
 End of File
*/
/*
 
Compiled on Apr 15 2021 at 11:32:42 UTC by XC8 version 2100
 Init PWM5 out RC5 = 25%

 Init NCO1 F=10Khz  U=  0  H=163 L=215 soit 0X00A3D7 

 Init My ADC 12bits entree EA1
 Start SMT1 pour 1 mesure ADC 12 bits 
 Stop SMT1   Nb Tics avec ultoa -> 2780 , soit  43 uS

 Test Presence I2C device (adressage 8 bits)

Compiled on Apr 15 2021 at 11:32:42 UTC by XC8 version 2100
 Init PWM5 out RC5 = 25%

 Init NCO1 F=10Khz  U=  0  H=163 L=215 soit 0X00A3D7 

 Init My ADC 12bits entree EA1
 Start SMT1 pour 1 mesure ADC 12 bits 
 Stop SMT1   Nb Tics avec ultoa -> 2780 , soit  43 uS

 Test Presence I2C device (adressage 8 bits)
 @ decimal # 120  soit @Device7bits = 0X3C OLED SSD1306
 @ decimal # 166  soit @Device7bits = 0X53 ADXL345 board
 @ decimal # 174  soit @Device7bits = 0X57 EEPROM 512bytes
 @ decimal # 208  soit @Device7bits = 0X68 RTC DS3231

Test IVT interrupt  18F27K42
 #1 Active sortie SQW=1Hz: 
 GO
 ok
 Etat pin RB1 = 2
 Re-Lecture et affichage des 7 Registres DS3231 
 Registre #26	
 Registre #30	
 Registre #11	
 Registre #04	
 Registre #15	
 Registre #04	
 Registre #21	
 Registre #00	

Jeudi  15 Avril  21 11H30M26S
15/04/21  11H30M28S    Cpt_T2    96   Cpt_T6=    3  Cpt IT_RBO=    0 PWM5 out=   9  % EA1= 1768  
 15/04/21  11H30M29S    Cpt_T2   127   Cpt_T6=    4  Cpt IT_RBO=    0 PWM5 out=  12  % EA1= 1771  
 15/04/21  11H30M30S    Cpt_T2   157   Cpt_T6=    5  Cpt IT_RBO=    0 PWM5 out=  15  % EA1= 1768  

  15/04/21  11H38M20S    Cpt_T2 14541   Cpt_T6=  464  Cpt IT_RBO=    0 PWM5 out=  20  % EA1= 1768  
 15/04/21  11H38M21S    Cpt_T2 14572   Cpt_T6=  465  Cpt IT_RBO=    0 PWM5 out=  23  % EA1= 1768  
 15/04/21  11H38M22S    Cpt_T2 14603   Cpt_T6=  466  Cpt IT_RBO=    0 PWM5 out=  26  % EA1= 1768  
 15/04/21  11H38M23S    Cpt_T2 14634   Cpt_T6=  467  Cpt IT_RBO=    0 PWM5 out=  29  % EA1= 1768  
 15/04/21  11H38M24S    Cpt_T2 14665   Cpt_T6=  468  Cpt IT_RBO=    0 PWM5 out=  32  % EA1= 1768  
 
U;15;04;21;11;41;04;#
Recu :
U;15;04;21;11;41;04;#

 MAJ RTC 
Ecriture DS3231... OK

 15/04/21  11H41M00S    Cpt_T2 14696   Cpt_T6=  469  Cpt IT_RBO=    0 PWM5 out=  36  % EA1= 1770  
 15/04/21  11H41M01S    Cpt_T2 14727   Cpt_T6=  470  Cpt IT_RBO=    0 PWM5 out=  39  % EA1= 1769  
 15/04/21  11H41M02S    Cpt_T2 14758   Cpt_T6=  471  Cpt IT_RBO=    0 PWM5 out=  42  % EA1= 1771  
*
 * */